'use strict';
var
   ws = require('sbis-node-ws'),
   Record = ws('WS.Data/Entity/Record'),
   SbisAdapter = ws('WS.Data/Adapter/Sbis'),
   RecordSet = ws('WS.Data/Collection/RecordSet'),
   cMerge = ws('Core/core-merge'),
   prepareGetRPCInvocationURL = ws('Deprecated/helpers/transport/prepareGetRPCInvocationURL'),
   BL = require('./blRequest'),
   Q = require('q'),
   config = require('video-utils/config').getConfig(),
   getLogger = require('./lib/getLogger'),
   moment = require('moment'),
   request = require('request').defaults({encoding: null});

const AUTH_STATUSES      = {
         NOT_AUTHORIZED: false,
         AUTHORIZED:     true,
         AUTHORIZING:    null
      },
      PG_DATETIME_FORMAT = 'YYYY-MM-DD HH:mm:ssZZ',
      PREVIEWER_URL = '/previewer',
      PHOTO_SIZE = 96,
      VIEWERS_COUNT = 20;

let _dataProvider;

var
   _getFieldType = function(rec, fieldName) {
      var
         recFormat = rec.getFormat(),
         fieldIndex = recFormat.getFieldIndex(fieldName),
         field = recFormat.at(fieldIndex);
      return field.getType();
   },
   _getRawValue = function(rec, fieldName) {
      var
         recFormat = rec.getFormat(),
         fieldIndex = recFormat.getFieldIndex(fieldName);
      return rec.getRawData().d[fieldIndex]
   };

var
   _each = function (obj, cb) {
      for (var i in obj) if (obj.hasOwnProperty(i)) {
         cb.call(this, obj[i], i, obj);
      }
   },
   _get = function (rec, field) {
      var result = null;
      if (rec.has(field)) {
         switch (_getFieldType(rec, field)) {
            case 'enum':
               result = rec.get(field).getAsValue();
               break;
            case 'datetime':
               // FIXME временное решение для правильного парсинга даты
               result = moment(_getRawValue(rec, field), PG_DATETIME_FORMAT).toDate();
               break;
            default:
               result = rec.get(field);
         }
         //typeof result === 'string' && (result = escapeHtml(result));
      }
      return result;
   },
   _processRecord = function (result) {
      return new Record({
         rawData: result,
         adapter: new SbisAdapter()
      });
   },
   _processRecordSet = function (result) {
      var rs = new RecordSet({
         rawData: result,
         adapter: new SbisAdapter()
      });
      return rs;
   },

   _auth = function (host, login, password) {
      var d = Q.defer();
      BL.doRequest({
         hostname: host,
         path: '/auth/service/',
         method: 'POST'
      }, {
         method: 'САП.АутентифицироватьРасш',
         data: {
            login: login,
            password: password
         }
      })
         .then(function (response) {
            if (response.error) {
               d.reject(response.error);
            }
            var
               rArray = response.result.d,
               sidFieldName = 'ИдСессии';

            for (var i = 0, len = rArray.length; i < len; i++) {
               if (rArray[i][0] === sidFieldName) {
                  d.resolve(rArray[i][1]);
               }
            }
         })
         .catch(function (err) {
            d.reject(err);
         });

      return d.promise;
   },


   proto = {
      getHost: function () {
         return this._options.host;
      },
      getSessionId: function () {
         if (this._options.sid) {
            return Promise.resolve(this._options.sid);
         } else{
            var self = this;
            return new Promise(function (resolve) {
               self.sidResolveFn = resolve;
            });
         }
      },
      auth: function () {
         var self = this;
         self._options.sid = null;
         self._authStatus = AUTH_STATUSES.AUTHORIZING;
         self.authTimer = self.authTimer ? self.authTimer * 2 : 1000;
         _auth(self._options.host, config.serviceUser.login, config.serviceUser.password)
            .then(function (sid) {
               self.logger.log('dataProvider', 'Авторизация на ' + self._options.host + ' прошла успешно');
               self._authStatus = AUTH_STATUSES.AUTHORIZED;
               self._options.sid = sid;
               typeof self.sidResolveFn === 'function' && self.sidResolveFn(sid);
               self.processQueue();
            })
            .catch(function (e) {
               self.logger.warn('dataProvider', 'Ошибка авторизации: ' + (e.message || e));
               setTimeout(function () {
                  self.logger.warn('dataProvider', 'Пробуем авторизоваться снова через ' + self.authTimer);
                  self.auth();
               }, self.authTimer)
            })

      },
      request: function (method, data, options) {
         options = options || {};

         var
            self = this,
            d = Q.defer(),
            request = {
               method: method,
               data: data,
               promise: d,
               options: options
            };

         if (self._authStatus == AUTH_STATUSES.AUTHORIZED) {
            BL
               .doRequest({
                  hostname: self._options.host,
                  path: options.serviceUrl || '/service/',
                  method: options.method || 'POST'
               }, {
                  method: method,
                  data: data,
                  sid: self._options.sid
               }, {
                  Cookie: `lang=${self._options.lang || 'ru-RU'}`
               })
               .then(function (response) {
                  self._processResponse(response, request);
               })
               .catch(function (e) {
                  d.reject(e.message || e);
               })
         } else {
            self.requestsQueue.push(request)
         }
         return d.promise;
      },
      _query: function (method, data, count) {
         var
            TYPES = {
               number: 'Число целое',
               string: 'Строка',
               boolean: 'Логическое'
            },
            filter = {
               s: [],
               d: []
            },
            nav = Number.isInteger(count) ? {
               "s": [{"n": "Страница", "t": "Число целое"}, {"n": "РазмерСтраницы", "t": "Число целое"}, {"n": "ЕстьЕще", "t": "Логическое"}],
               "d": [0, count, false]
            } : null;

         if (data.d && data.s) {
            filter = data
         } else {
            _each(data, function (value, param) {
               filter.s.push({
                  n: param,
                  t: TYPES[typeof value]
               });
               filter.d.push(value)
            });
         }

         return this.request(method, {
            'ДопПоля': [],
            'Навигация': nav,
            'Сортировка': null,
            'Фильтр': filter
         });
      },
      formatRecord: _processRecord,
      _processResponse: function (response, request) {
         var
            self = this,
            err = response.error,
            result = response.result != null ? response.result : {};

         if (err) {
            if (/not authorized/.test(err.toString().toLowerCase())) {
               self.requestsQueue.push(request);
               if (self._authStatus != AUTH_STATUSES.AUTHORIZING) {
                  self.auth();
               }
            } else if (err.message == 'У вас нет прав для просмотра этого вебинара.') {
               err.type = 'forbidden';
               request.promise.reject(err);
            } else {
               request.promise.reject(['error: ', err.message || err, '; method: ', request.method, '; data: ', JSON.stringify(request.data)].join(''));
            }
         } else {
            if (result._type === 'recordset') {
               request.promise.resolve(_processRecordSet(result));
            } else if (result._type === 'record') {
               request.promise.resolve(_processRecord(result));
            } else {
               request.promise.resolve(result);
            }
         }
      },
      processQueue: function () {
         var self = this;
         if (self.requestsQueue.length) {
            var step = self.requestsQueue.shift();
            BL.doRequest({
               hostname: self._options.host,
               path: step.options.serviceUrl || '/service/',
               method: step.options.method || 'POST'
            }, {
               method: step.method,
               data: step.data,
               sid: self._options.sid
            })
               .then(function (response) {
                  self._processResponse(response, step);
                  self.processQueue();
               })
               .catch(function (e) {
                  step.promise.reject(e.message || e);
               })
         }
      },
      getCpsUserIdByClientIdFaceId: function (cfgOrArray) {
         if (!(cfgOrArray instanceof Array)) {
            cfgOrArray = [cfgOrArray];
         }
         cfgOrArray = cfgOrArray.map(function (cfg) {
            return [cfg.clientId, cfg.faceId];
         });
         return this.request('СервисПрофилей.ПолучитьИдЛиц', {
            'Лица': {
               's': [{'n': 'Клиент', 't': 'Число целое'}, {'n': 'Лицо', 't': 'Число целое'}],
               'd': cfgOrArray
            }
         })
            .then(function (recordSet) {
               var
                  record = recordSet.at(0),
                  result = null;

               if (record && record.has('Идентификатор')) {
                  result = record.get('Идентификатор');
               }

               return result;
            });
      },
      getCpsUserIdByFaceId: function (faceId) {
         return this.request('СервисПрофилей.ИдентификаторЛица', {
            'Лицо': faceId
         });
      },
      getFaceIdByCpsUserId: function (cpsUserId) {
         return this.request('СервисПрофилей.ПолучитьЛицаПоИд', {
            'Персоны': [cpsUserId]
         })
            .then(function (recordSet) {
               var
                  record = recordSet.at(0),
                  result = null;

               if (record && record.has('Лицо')) {
                  result = record.get('Лицо');
               }

               return result;
            });
      },
      _formatUserRecord: function (record, cpsUserId, requestExtraFields) {
         var
            d = Q.defer(),
            user, postRecord,
            recWrapper = new RecordWrapper(record),
            postRecordWrapper;

         requestExtraFields = (typeof requestExtraFields == 'undefined') ? true : requestExtraFields;

         if (record && recWrapper.get('ЕстьПользователь')) {
            user = {};
            user.cpsUserId = recWrapper.get('Персона');
            user.clientId = recWrapper.get('Клиент');
            user.faceId = recWrapper.get('Лицо');
            user.surname = recWrapper.get('Фамилия');
            user.firstname = recWrapper.get('Имя');
            user.middlename = recWrapper.get('Отчество');
            user.photoId = recWrapper.get('Фото') || '';
            user.photoIdMini = recWrapper.get('ФотоМини')  || '';
            user.shortname = [user.surname, user.firstname && (user.firstname.charAt(0) + '.')].join(' ');
            user.name = [user.surname, user.firstname].join(' ');
            postRecord = recWrapper.get('Должности').at(0);
            if (postRecord) {
               var
                  postRecordWrapper = new RecordWrapper(postRecord),
                  org = postRecordWrapper.get('ОрганизацияНазвание'),
                  dep = postRecordWrapper.get('ПодразделениеНазвание');

               user.post = postRecordWrapper.get('ДолжностьНазвание');
               user.departmentExternal = '';
               if (org) {
                  user.departmentExternal += org;
                  if (dep) {
                     user.departmentExternal += ' / ' + dep;
                  }
               } else {
                  user.departmentExternal = dep;
               }
            }
            // на my.sbis.ru нет объекта БЛ - ЧастноеЛицо
            if (requestExtraFields && user.faceId && !/my\.sbis\.ru/.test(this._options.host)) {
               this.request('ЧастноеЛицо.ПодробнаяИнформация', {
                  'ИдО': String(user.faceId)
               })
                  .then(function (faceRS) {
                     var faceRec, faceRecWrapper;
                     if (faceRS && (faceRec = faceRS.at(0))) {
                        faceRecWrapper = new RecordWrapper(faceRec);
                        user.departmentInternal = faceRecWrapper.get('ПодразделениеПолныйПуть');
                        user.userId = faceRecWrapper.get('Пользователь');
                     }
                  })
                  .finally(d.resolve.bind(d, user));
            } else {
               d.resolve(user);
            }

         } else {
            user = {
               cpsUserId: cpsUserId,
               name: 'Гость',
               shortname: 'Гость',
               photoId: '',
               guest: true,
               departmentExternal: ''
            };
            d.resolve(user);
         }

         return d.promise;
      },
      _getPersonsRequest: function (persons) {
         return this.request('СервисПрофилей.ПолучитьЛицаПоИд', {
            'Персоны': persons
         });
      },
      /**
       * Получить пользователей по идентификаторам сериса профилей
       * @params{array} persons идентификаторы сервиса профилей
       */
      getUsersByCpsUserId: function (persons, requestExtraFields) {
         var self = this, promises = [], filter;
         if (typeof requestExtraFields === 'object') {
            filter = function(id) {
               return requestExtraFields.hasOwnProperty(id);
            };
         } else {
            filter = function(id) {
               return requestExtraFields;
            };
         }
         return this._getPersonsRequest(persons)
            .then(function (recordSet) {
               recordSet.each(function (record, idx) {
                  promises.push(self._formatUserRecord(record, persons[idx], filter(persons[idx])));
               });
               return Q.all(promises);
            });
      },
      /**
       * - Получить пользователя по идентификаторам сериса профилей
       * @params{string} cpsUserId идентификатор сервиса профилей
       * - name (surname firstname)
       * - firstname
       * - middlename
       * - surname
       * - shortname(surname firstname.charAt(0).)
       * - departmentInternal
       * - departmentExternal
       * - post
       * - clientId
       * - faceId
       * - cpsUserId
       * - userId
       */
      getUserByCpsUserId: function (cpsUserId) {
         return this._getPersonsRequest([cpsUserId])
            .then(function (recordSet) {
               return this._formatUserRecord(recordSet.at(0), cpsUserId);
            }.bind(this));
      },

      getUserByFaceId: function (faceId) {
         return this.getCpsUserIdByFaceId(faceId).then(this.getUserByCpsUserId.bind(this));
      },
      /**
       * Создать чат по вебинару
       * @returns {string} идентификатор темы
       */
      createChat: function (docUuid, name, authorUuid) {
         var record = new Record({
            rawData: {
               'Документ': docUuid,
               'ПубличныйДокумент': true,
               'Название': name,
               'Автор': authorUuid
            }
         });
         return this.request('Чат.Создать', {
            'ИмяМетода': 'Чат.СоздатьПубличный',
            'Фильтр': record.toJSON()
         })
            .then(function (result) {
               return result.has('Тема') ? result.get('Тема') : null;
            });
      },
      checkRights: function (webinarUuid) {
         return this.request('Meeting.ReadWebinar', {
            'ИдО': webinarUuid,
            'ИмяМетода': ''
         })
      },
      /**
       * Получить информацию о вебинаре
       * @param {string} webinarUuid идентификатор вебинара
       */
      getWebinarInfo: function (webinarUuid) {
         var
            RETURN_FIELDS = {
               name: 'Name',
               description: 'Description',
               shortDescription: 'ShortDescription',
               time: 'DateTimeStart',
               endTime: 'DateTimeEnd',
               endTimeReal: 'DateTimeEndReal',
               pauseBeginTimeReal: 'DateTimePauseBegin',
               pauseEndTimeReal: 'DateTimePauseEnd',
               startTimeReal: 'DateTimeStartReal',
               uuid: 'Id',
               groupUuid: 'Group',
               status: 'TranslationState',
               folderId: 'SbisDiskFolder',
               author: 'Author',
               chatUuid: 'DefaultChat',
               docType: 'Type',
               webinarType: 'AccessType',
               responsible: 'Responsible',
               isDeleted: 'IsDeleted',
               state: 'State'
            };

         return this.request('Meeting.ReadWebinar', {
            'ИдО': webinarUuid,
            'ИмяМетода': ''
         })
            .then(function (record) {
               var result = {},
                  recordWrapper = new RecordWrapper(record);

               _each(RETURN_FIELDS, function (field, key) {
                  result[key] = recordWrapper.get(field);
               });

               result.landing = JSON.parse(record.get('Landing') || '{}') || {};
               result.time = +result.time;
               result.endTime = +result.endTime;

               return this.getUserByCpsUserId(result.author)
                  .then(function (author) {
                     result.author = {
                        client: author.clientId,
                        face: author.faceId,
                        person: author.cpsUserId
                     };
                     return result;
                  });
            }.bind(this)).catch(
               function (e) {
                  throw e
               }
            );

      },
      /**
       * Получить идентификаторы всех выступащих (без исключения) и зрителей (подтвердивших участие)
       */
      getUsersPlainList: function (webinarUuid, groupUuid, viewersCount) {
         return Q.all([this._getWebinarUsers(webinarUuid, groupUuid, true), this._getWebinarUsers(webinarUuid, groupUuid, false, viewersCount)])
            .then(function(res) {
               var
                  presentersRecordSet = res[0],
                  viewersRecordSet = res[1],
                  presenters = [],
                  viewers = [];

               presentersRecordSet.each(function (record) {
                  var recordWrapper = new RecordWrapper(record);
                  presenters.push(recordWrapper.get('Subscriber'))
               });

               viewersRecordSet.each(function (record) {
                  var recordWrapper = new RecordWrapper(record);
                  viewers.push(recordWrapper.get('Subscriber'))
               });

               return {
                  presenters: presenters,
                  viewers: viewers,
                  viewersCount: viewersRecordSet._$rawData.n
               };
            });
      },
      /**
       * Получить список участников вебинара
       * @param {string} webinarUuid идентификатор вебинара
       * @param {string} groupUuid идентификатор группы соц. сети
       * @param {boolean} presentersOrViewers вернуть только выступащих или только зрителей
       * @param {number} count количество пользователей
       *
       */
      _getWebinarUsers: function (webinarUuid, groupUuid, presentersOrViewers, count) {
         var filter = {
            'ИдДокумента': webinarUuid,
            'Channel': groupUuid,
            'IncludeFolders': false,
            'Inheritance': true,
            'Speaker': presentersOrViewers
         };
         if (!presentersOrViewers) {
            filter['MeetingMember'] = true;
         }
         return this._query('Subscriber.GetMembers', filter, count);
      },
      /**
       * Получить потенциальных участников вебинара
       * @param {string} webinarUuid идентификатор вебинара
       * @param {string} groupUuid идентификатор группы соц. сети
       */
      getWebinarViewers: function (webinarUuid, groupUuid) {
         var self = this;
         return this._getWebinarUsers(webinarUuid, groupUuid, false)
            .then(function (recordSet) {
               var persons = [];
               recordSet.each(function (record) {
                  var recordWrapper = new RecordWrapper(record);
                  persons.push(recordWrapper.get('Subscriber'));
               });
               return self.getUsersByCpsUserId(persons, false)
                  .then(function (users) {
                     var result = {};
                     users.forEach(function (user) {
                        result[user.cpsUserId] = user;
                     });
                     return result;
                  });
            });
      },
      /**
       * Получить выступающих вебинара
       * @param {string} webinarUuid идентификатор вебинара
       * @param {string} groupUuid идентификатор группы соц. сети
       */
      getWebinarPresenters: function (webinarUuid, groupUuid) {
         var self = this;
         return this._getWebinarUsers(webinarUuid, groupUuid, true)
            .then(function (recordSet) {
               var persons = [];
               recordSet.each(function (record) {
                  var recordWrapper = new RecordWrapper(record);
                  persons.push(recordWrapper.get('Subscriber'));
               });
               return self.getUsersByCpsUserId(persons)
                  .then(function (users) {
                     var result = {};
                     users.forEach(function (user) {
                        user.presenter = true;
                        result[user.cpsUserId] = user;
                     });
                     return result;
                  });
            });
      },
      /**
       * Получить выступающих и потенциальных участников вебинара
       * @param {string} uuid идентификатор вебинара
       * @param {string} groupUuid идентификатор группы соц. сети
       * @param {object} params параметры запроса
       */
      getWebinarUsers: function (uuid, groupUuid, params) {
         var
            self = this,
            presenters = {},
            viewersCount,
            params = params || {};
         return this.getUsersPlainList(uuid, groupUuid, VIEWERS_COUNT)
            .then(function (users) {
               var maxUsersCount = params.hasOwnProperty('maxUsersCount') ? params.maxUsersCount : users.viewersCount;
               viewersCount = users.viewersCount;
               users.presenters.forEach(function(uid) {
                  presenters[uid] = 1;
               });
               return self.getUsersByCpsUserId(users.viewers.slice(0, maxUsersCount).concat(users.presenters), presenters);
            })
            .then(function(users) {
               var result = {};
               users.forEach(function (u) {
                  if (presenters.hasOwnProperty(u.cpsUserId)) {
                     u.presenter = true;
                  }
                  result[u.cpsUserId] = u;
               });
               return {users: result, viewersCount: viewersCount};
            });
      },
      /**
       * Прочитать вебинар и список участников, включая ведущих
       * @param {string} uuid идентификатор вебинара
       */
      getWebinar: function (uuid, params) {
         return this.getWebinarInfo(uuid)
            .then(function (webinar) {
               return this.getWebinarUsers(uuid, webinar.groupUuid, params)
                  .then(function (users) {
                     webinar.viewersCount = users.viewersCount;
                     return {
                        info: webinar,
                        users: users.users
                     }
                  });
            }.bind(this));
      },
      attachRecord: function (webinarId, fileId) {
         return this.request('Документ.ДобавитьВД', {
            'ИД': fileId,
            'Документ': parseInt(webinarId, 10),
            'фБезНазвания': false,
            'фЭтоСкан': false
         });
      },
      changeStatus: function (webinarId, status) {
         return this.request('Meeting.ChangeBroadcastingState', {
            id: webinarId,
            state: status
         });
      },

      updateWebinar: function (webinar_rec) {
         return this.request('Meeting.UpdateWebinarFields', {
            webinar_rec: webinar_rec
         });
      },

      /**
       * Возвращает информацию о активности пользователя
       * @param{Array} uuid - идентификатор персоны (идентификатор сервиса профилей)
       * @return{String} activity - unix timestamp последней активности:
       */
      getLastActivity: function (uuid) {
         return this.request('Местоположение.GetLastActivityByUUID', {
            UUIDs: [uuid]
         }, {serviceUrl: '/activity/service/'})
      },

      //Получить представителей контраганта по идентификатору контрагента
      //* @param{Number} id - идентификатор контрагента (идентификатор лица)
      getRepresentatives: function (id) {
         var self = this;
         return self.request('Контрагент.Прочитать', {
            "ИдО": id,
            "ИмяМетода": ''
         })
            .then(function (rs) {
               var res = {};
               res["ИНН"] = rs.has('ИНН') ? rs.get('ИНН') : null;
               res["ИдентификаторСПП"] = rs.has('ИдентификаторСПП') ? rs.get('ИдентификаторСПП') : null;
               return self._query('СервисПрофилей.ПоискКонтактовОрганизацийСписок', {
                  "ПлоскийСписок": true,
                  "ИНН": res["ИНН"],
                  "ИдентификаторСПП": res["ИдентификаторСПП"]
               }).then(function (recordSet) {
                  var res = [];
                  recordSet.each(function (record) {
                     var
                        recordWrapper = new RecordWrapper(record),
                        uuid = recordWrapper.get('Идентификатор');

                     uuid && res.push(uuid);
                  });
                  return res;
               })
            })
            .catch(function (err) {
               self.logger.error('getRepresentatives', err);
               throw err;
            })
      },

      getUserInfo: function (personsId, photoSizeList = []) {
         if (this._options.old) {
            return this.getUserInfoOld(personsId, photoSizeList);
         }
         return this.request('СервисПрофилей.СписокПерсонJSON', {
            'Filter': {
               'Персоны': personsId,
               'ТолькоФИО': false,
               'СортироватьПоФИО': false,
               'UniversalPhotoIds': true,
            }
         }).then(result => {
            const info                = {},
                  getAdditionalInfoPromiseList = [],
                  self                = this;

            result && _each(result.data, function(person, index) {
               if (!person['ЕстьПользователь']) return;

               const personId      = person['Персона'],
                     photoUrl      = prepareGetRPCInvocationURL('СервисПрофилей', 'ФотоПоИд', {
                        'Ид': person['Фото']
                     }),
                     post          = person && person['Должности'] && person['Должности'][0];

               info[personId] = {
                  clientId:         person['Клиент'],
                  faceId:           person['Лицо'],
                  name:             `${person['Фамилия'] || ''} ${person['Имя'] || ''}`,
                  department:       (post && post['ПодразделениеНазвание']) || '',
                  clientName:       (post && post['ОрганизацияНазвание']) || '',
                  uuid:             personId,
                  post:             (post && post['ДолжностьНазвание']) || '',
                  videocallAllowed: true,
                  photoUrl
               };

               getAdditionalInfoPromiseList.push(new Promise(resolve => {
                  request.get(`${self._options.host}${PREVIEWER_URL}/r/${photoSizeList[index] || PHOTO_SIZE}${photoUrl}`, {
                     headers: {
                        Cookie: `sid=${self._options.sid}`
                     }
                  }, (err, response, body) => {
                     info[personId].photo = `data:image/png;base64,${new Buffer(body).toString('base64')}`;
                     resolve();
                  });
               }));
               getAdditionalInfoPromiseList.push(self.request('ProfileServicePerson.Read', {
                  'Person': personId
               }).then((res)=> {
                  info[personId].videocallAllowed = res.get('VideocallAllowed');
               }));
            });

            return Q.all(getAdditionalInfoPromiseList).timeout(1000).then(() => info).catch(() => info);
         });
      },
      getUserInfoOld: function (personsId, photoSizeList = []) {
         return this.request('СервисПрофилей.СписокПерсонJSON', {
            'Filter': {
               'Персоны': personsId,
               'ТолькоФИО': false,
               'СортироватьПоФИО': false,
               'UniversalPhotoIds': true,
            }
         }).then(result => {
            const info                = {},
                  getPhotoPromiseList = [],
                  self                = this;

            result && _each(result.data, function(person, index) {
               if (!person['ЕстьПользователь']) return;

               const personId      = person['Персона'],
                     photoUrl      = prepareGetRPCInvocationURL('СервисПрофилей', 'ФотоПоИд', {
                        'Ид': person['Фото']
                     }),
                     post          = person && person['Должности'] && person['Должности'][0];

               info[personId] = {
                  clientId:   person['Клиент'],
                  faceId:     person['Лицо'],
                  name:       `${person['Фамилия'] || ''} ${person['Имя'] || ''}`,
                  department: (post && post['ПодразделениеНазвание']) || '',
                  clientName: (post && post['ОрганизацияНазвание']) || '',
                  uuid:       personId,
                  post:       (post && post['ДолжностьНазвание']) || '',
                  photoUrl
               };

               getPhotoPromiseList.push(new Promise(resolve => {
                  request.get(`${self._options.host}${PREVIEWER_URL}/r/${photoSizeList[index] || PHOTO_SIZE}${photoUrl}`, {
                     headers: {
                        Cookie: `sid=${self._options.sid}`
                     }
                  }, (err, response, body) => {
                     info[personId].photo = `data:image/png;base64,${new Buffer(body).toString('base64')}`;
                     resolve();
                  });
               }));
            });

            return Q.all(getPhotoPromiseList).timeout(1000).then(() => info).catch(() => info);
         });
      },
      getDocumentTitle: function (docGUID) {
         if (!docGUID) {
            return Promise.resolve('');
         } else {
            return this.request('Документ.КрасивыеНазвания', {
               'Документы': [docGUID]
            })
                  .then(docsRS => {
                     const docRec = docsRS && docsRS.at(0);
                     if (!docRec) {
                        return null;
                     }

                     const record = new RecordWrapper(docRec);
                     return record.get('НеДоступен') ? null : record.get('КрасивоеНазвание');
                  });
         }
      },
      getGroup: function(groupUuid) {
         if (groupUuid) {
            return this.request('Group.Read', {
               'ИдО': groupUuid,
               'ИмяМетода': 'List'
            });
         } else {
            return Promise.resolve();
         }
      },

      attendUser: function(webinarId, personId) {
         return this.request('Meeting.AttendUser', {
            WebinarId: webinarId,
            ProfileId: personId
         });
      }
   };

function DataProvider(options, _logger) {
   this.logger = getLogger(_logger);
   BL.setLogger(this.logger);
   this.requestsQueue = [];
   if (!options || !options.host || !options.sid) {
      this._options = {
         host: config.serviceUser.address
      };
      this._authStatus = AUTH_STATUSES.NOT_AUTHORIZED;
      this.auth();
   } else {
      this._authStatus = AUTH_STATUSES.AUTHORIZED;
      this._options = options;
   }
}

DataProvider.getDataProvider = function (logger) {
   if (_dataProvider) {
      return _dataProvider;
   } else {
      return _dataProvider = new DataProvider(null, logger);
   }
};

cMerge(DataProvider.prototype, proto);

function RecordWrapper(record) {
   this._record = record;
   this._recFormat = {};
   this._index = {};
   this._initialize();
}

RecordWrapper.prototype._initialize = function() {
   var formats;
   if (this._record) {
      formats = this._record._$rawData.s;
      formats.forEach(function(field, index) {
         if (field.t === 'Дата и время') {
            this._recFormat[field.n] = 'datetime';
         } else if (typeof field.t === 'object') {
            this._recFormat[field.n] = 'enum';
         } else {
            this._recFormat[field.n] = field.t;
         }
         this._index[field.n] = index;
      }.bind(this));
   }
};

RecordWrapper.prototype._getFieldType = function(fieldName) {
   return this._recFormat[fieldName];
};

RecordWrapper.prototype._getFieldValue = function(fieldName) {
   return this._record._$rawData.d[this._index[fieldName]];
};

RecordWrapper.prototype.get = function(field) {
   var result = null;
   if (this._record.has(field)) {
      switch (this._getFieldType(field)) {
         case 'enum':
            result = this._record.get(field).getAsValue();
            break;
         case 'datetime':
            result = moment(this._getFieldValue(field), PG_DATETIME_FORMAT).toDate();
            break;
         default:
            result = this._record.get(field);
      }
   }
   return result;
};

module.exports = DataProvider;
