const
   os = require('os'),
   bl = require('sbis3-bl-request');

module.exports = {
   doRequest: function(connection, reqParams, additionalHeaders) {
      additionalHeaders = additionalHeaders || {};

      let ext = {
         'User-Agent': [os.hostname(), global.sbisVersion].join(' ')
      };

      if (reqParams.sid){
         ext['X-SBISSessionID'] = reqParams.sid;
      }

      Object.keys(additionalHeaders).forEach(function(key){
         ext[key] = additionalHeaders[key];
      });

      let timeout = 0;

      const doRec = function() {
         return bl.doRequest(Object.assign({}, connection), Object.assign({}, reqParams), ext).catch(function(e) {
            if(e.code === 'ECONNRESET') {
               return reSendRequest().then(res => {
                  timeout = 0;
                  return res;
               });
            } else {
               throw e;
            }
         });
      };

      const reSendRequest = function() {
         if(!timeout) {
            timeout = 500;
            return doRec();
         } else {
            timeout *= 2;
            return new Promise(function(resolve) {
               setTimeout(function() {
                  resolve(doRec());
               }, timeout);
            })

         }

      };
      const startTime = Date.now();
      return doRec().then(res => {
         if (this.logger) {
            this.logger.info('BL.doRequest', `\n${JSON.stringify(connection)}\n${JSON.stringify(reqParams)}\n${JSON.stringify(additionalHeaders)}\n${Date.now() - startTime}ms`)
         }
         return res;
      }).catch(err => {
         if (this.logger) {
               this.logger.error('BL.doRequest', `\n${JSON.stringify(connection)}\n${JSON.stringify(reqParams)}\n${JSON.stringify(additionalHeaders)}\n${Date.now() - startTime}ms\n${err.message || err}`)
         }
         throw err;
      });
   },
   setLogger: function (logger) {
      this.logger = logger;
   }
};