var
   path = require('path'),
   extend = require('util')._extend,
   logger = require('log4js').getLogger(),
   q = require('q'),
   os = require('os'),
   configuration = {},
   initialized = false,
   dCloudConfig;

var TIMER_END = 7200000,// Лимит времени, которое требуется на установку соединения (2 ч)
    FIRST_TIMER = 60000; // Первоначальное время (1 мин)

var humanize = {
   db: {
      address    : 'СервисВидеосвязи.БД.Адрес',
      dbname     : 'СервисВидеосвязи.БД.Имя',
      username   : 'СервисВидеосвязи.БД.Пользователь',
      password   : 'СервисВидеосвязи.БД.Пароль'
   },
   hosts: {
      allowed    : 'СервисВидеосвязи.РазрешенныеСервисы'
   },
   profiles: {
      address    : ['АдресСервисаПрофилей', "profiles.Адрес"]
   },
   rabbit: {
      clientAddress    : 'Ядро.Асинхронные сообщения.БрокерыПодпискиНаКлиентскиеСобытия',
      serverAddress    : 'Ядро.Асинхронные сообщения.БрокерыПодпискиНаСобытия',
      exchanges        : 'СервисВидеосвязи.РазрешенныеСервисы'
   },
   redis: {
      state: {
         address : 'СервисВидеосвязи.Хранилище.Адрес'
      }
   },
   log: {
      address    : 'Центр логирования.Адрес'
   },
   admin: {
      address    : 'СервисАдминистрирования.Адрес'
   },
   serviceUser: {
      address        : 'СервисВидеосвязи.СлужебныйПользователь.Адрес',
      authentication : 'СервисВидеосвязи.СлужебныйПользователь.Аутентификация',
      login          : 'СервисВидеосвязи.СлужебныйПользователь.Логин',
      password       : 'СервисВидеосвязи.СлужебныйПользователь.Пароль'
   }
};
// Дополнительная конфигурация (не из обалака), для параметров, передаваемых при запуске сервиса [по умолчанию]
var extraDefault = {
   port: 81,
   cloudLogger: false
};

var normalize = function(name, value) {
   switch(name) {
      case 'СервисВидеосвязи.РазрешенныеСервисы':
         return value.split(';').map(function(elem){return elem.trim()}).filter(function (elem) {return !!elem});
      default:
         return value;
   }
};

module.exports = {
   /**
    * Инициализовать конфигурацию
    * @param{Object} args — дополнительная конфигурация (не из облака)
    */
   init: function(args) {
      var timer = FIRST_TIMER,
          deferred = q.defer(),
            // Объединим основную и дополнительную конфигурацию
            mergeExtraConfig = function() {
               configuration.extra = {};
               for (var cfg in extraDefault) {
                  if (!extraDefault.hasOwnProperty(cfg)) continue;
                  configuration.extra[cfg] = args[cfg] || extraDefault[cfg];
               }
            },
            eachRecursive = function(destination, source, getFromServices) {
               var _get = function (param) {
                  if (getFromServices) {
                     return cloudData.get('__SBIS_SERVICE_ENDPOINTS__')[param];
                  } else {
                     return cloudData.get(param);
                  }
               };

               for (var prop in source) {
                  if (!source.hasOwnProperty(prop)) continue;
                  if (typeof source[prop] === 'object' && source[prop] !== null && !(source[prop] instanceof Array)) {
                     if (!destination.hasOwnProperty(prop)) destination[prop] = {};
                     eachRecursive(destination[prop], source[prop], prop === 'services');
                  } else {
                     if (source[prop] instanceof Array) {
                        source[prop].every(function (sourceProp) {
                           var param = _get(sourceProp);
                           if (param) {
                              cloudParam = param;
                              return false;
                           } else {
                              return true;
                           }
                        });
                     } else {
                        cloudParam = _get(source[prop]);
                     }
                     if (!cloudParam) {
                        errors.indexOf(source[prop]) === -1 && errors.push(source[prop]);
                     } else {
                        cloudParam = normalize(source[prop], cloudParam);
                     }
                     destination[prop] = cloudParam;
                  }
               }
            },
            cloudData, cloudParam, errors = [];
      args = args || {};
      if (!args.cloudConfigPath && args.debugConfigPath) {
         // загружаем debug-config
         try {
            configuration = require(path.join(args.debugConfigPath, 'debug-config.json'));
            mergeExtraConfig();
            initialized = true;
            deferred.resolve(configuration);
         } catch (err) {
            logger.error('Configuration', (err.stack || err));
            deferred.reject(err);
         }

      } else if (args.cloudConfigPath) {
         // загружаем конфиг из облака
         var cloudCfg = require(path.join(args.cloudConfigPath, 'cloud.json'));

         if (cloudCfg.url && cloudCfg.url.indexOf('private') !== -1) {
            delete cloudCfg.login;
            delete cloudCfg.password;
            cloudCfg.cat = '/service';
         }

         if (args.extra) {
            extend(humanize, args.extra);
            delete args.extra;
         }

         cloudCfg.name = cloudCfg.name || args.sysName;
         cloudCfg.hostname = cloudCfg.hostname || args.hostname || os.hostname();

         /**
          * Переподключение облака
          */
         var reloadCloud = function () {

               dCloudConfig = require('sbis3-cloud-config').init(cloudCfg);

               dCloudConfig.once('init', function (_cloudData) {
                  cloudData = _cloudData;
                  eachRecursive(configuration, humanize);
                  if (args.services) {
                     eachRecursive(configuration, args.services, true);
                  }
                  if (args.exclude instanceof Array && args.exclude.length) {
                     args.exclude.forEach(function (param) {
                        var index = errors.indexOf(param);
                        if (index !== -1) {
                           errors.splice(index, 1);
                        }
                     });
                  }
                  if (errors.length) {
                     throw new Error('В облаке не заданы параметры конфигурации: ' + errors.join(', '));
                  }
                  if (args.optional) {
                     eachRecursive(configuration, args.optional);
                  }

                  mergeExtraConfig();
                  initialized = true;
                  deferred.resolve(configuration);
               });

               dCloudConfig.once('error', function (err) {
                  logger.error('Configuration', (err.stack || err));
                  if(timer >= TIMER_END){
                     deferred.reject(err);
                  }
                  else{
                     logger.warn('Configuration', 'Пробуем снова ...');
                     setTimeout(reloadCloud, timer);
                     timer *= 2;
                  }
               });
         };
         reloadCloud();
      } else {
         var err = 'Необходимо передать cloudConfigPath или debugConfigPath';
         logger.error('Configuration', err);
         deferred.reject(new Error(err));
      }
      return deferred.promise;
   },
   /**
    * Получить всю конфигурацию
    */
   getConfig: function() {
      return extend({}, configuration);
   },
   /**
    * Получить конфигурацию по ключу
    */
   get: function(key) {
      var param = configuration[key];
      if (typeof param === 'object' && param !== null) {
         return extend({}, configuration[key]);
      } else {
         return configuration[key];
      }
   },
   /**
    * Обновить конфигурацию
    */
   update: function(cfg) {
      // TODO: сделать итеративное обновление
      configuration = cfg;
   },
   getCloudConfig: function () {
      return dCloudConfig;
   }
};