const fs = require('fs');
const path = require('path');
const getLogger = require('./lib/getLogger');

const getFiles = function(dir) {
   const result = [];

   fs.readdirSync(dir).forEach(fileName => {
      const filePath = path.join(dir, fileName);
      const files = fs.statSync(filePath).isDirectory()
         ? getFiles(filePath)
         : [filePath];
      result.push(...files);
   });

   return result;
};

module.exports = {
   /**
    * Инициализация методов локализации
    * @param {string} dictDir путь до папки со словарями
    * @param {{error: function}} [logger] логгер для записи ошибкок перевода
    */
   init(dictDir, logger) {
      logger = getLogger(logger);
      let lang = 'ru-RU';
      const dict = {};

      // Считываем все доступные словари
      const files = getFiles(dictDir);
      files.forEach(file => {
         const fileName = path.basename(file, '.json');
         dict[fileName] = require(file);
      });

      const setLang = (newLang) => {
         newLang = newLang || 'ru-RU';

         if (newLang !== 'ru-RU' && !dict[newLang]) {
            logger.error('Нет словаря для языка: ' + newLang);
            return;
         }

         lang = newLang;
      };

      const rk = function(key) {
         if (lang === 'ru-RU') {
            return key;
         }

         const translate = dict[lang][key];
         if (!translate) {
            logger.error('Не найден перевод для фразы: ' + key);
            return key;
         }

         return translate;
      };

      return { rk, setLang }
   }
};