var
   path = require('path'),
   logger = require('./../logger/logger'),
   dot = require('dot'),
   fs = require('fs'),
   pgClient = require('./../db/pgManager')();

dot.templateSettings.strip = false;

var
   pgQuery = function (file, type, params, values, queryName) {
      var
         sql = fs.readFileSync(file, 'utf8'),
         query = dot.template(sql)(params);
      return pgClient[type](query, values, queryName)
         .then(function(result) {
            return result ? (result.rows || []) : null;
         })
         .catch(function (err) {
            logger.warn('DataBase', 'Ошибка при работе с базой данных: Запрос: ' + query + '; Ошибка: ' + err);
            return err.message || 'Внутренняя ошибка сервера'
         });
   },

   prepareOptionsQuery = function (options, type, keyName) {
      var
         columns = [], values = [], key;
      if(type === 'update' && keyName && options instanceof Array && options[0]) {
         key = options[0][keyName];
      }
      options.forEach(function (row) {
         var
            rowValues = [],
            colValues = [];
         for (var name in row) {
            if (row.hasOwnProperty(name)) {
               colValues.push(name);
               if (type === 'copy' && (row[name] === null || row[name] === undefined)) {
                  row[name] = 'NULL';
               }
               rowValues.push(row[name]);
            }

         }
         values.push(type === 'copy' ? (rowValues.join('\t') + '\n') : rowValues);
         columns.push(type === 'copy' ? colValues.join(',') : colValues);
      });
      return {
         columns: type === 'copy' ? columns[0] : columns,
         values: type === 'copy' ? values.join('') : values,
         key: key
      }
   };

module.exports = {
   insert: function (file, queryName, options) {
      if (options && options.length) {
         options = prepareOptionsQuery(options, 'insert');
         options.columns.forEach(function (row, index) {
            return pgQuery(file, 'query', row, options.values[index], queryName);
         });
      }
   },

   update: function (file, queryName, options, keyName) {
      if (options && options.length) {
         options = prepareOptionsQuery(options, 'update', keyName);
         options.columns.forEach(function (row, index) {
            return pgQuery(
               file,
               'query',
               {
                  columns: row,
                  key: options.key
               },
               options.values[index],
               queryName
            );
         });
      }
   },

   copy: function (file, queryName, options) {
      if (options && options.length) {
         options = prepareOptionsQuery(options, 'copy');
         pgQuery(file, 'copy', options.columns, options.values, queryName)
      }
   },
   select: function (file, queryName, options) {
      return pgQuery(file, 'query', options, null, queryName);
   }
};
