var
      log4js = require('log4js'),
      Q = require('q'),
      shutdownManager = require('./shutdownManager'),
      sbisCloudLogger,
      consoleLogger,
      cloudLogger;

methodsForExport = {
   end: function (cb) {
      sbisCloudLogger && sbisCloudLogger.end(cb);
   },
   error: function (tag, msg, onlyCloud) {
      this._log(1, tag, msg, onlyCloud);
   },
   info: function (tag, msg, onlyCloud) {
      this._log(2, tag, msg, onlyCloud);
   },
   log: function (tag, msg, onlyCloud) {
      this._log(3, tag, msg, onlyCloud);
   },
   warn: function(tag, msg, onlyCloud) {
      this._log(4, tag, msg, onlyCloud);
   },
   _log: function (level, tag, msg, onlyCloud) {
      if (!msg) {
         msg = tag;
         tag = undefined;
      }

      if (msg && (typeof msg === 'string' || msg instanceof Error)) {
         var text = '';

         if (tag) {
            text += "[" + (tag || '') + "]\t";
         }
         text = text + (msg && msg + "") || '';
      }
      switch (level) {
         case 1:
            cloudLogger && cloudLogger.error(text);
            if (!onlyCloud) {
               consoleLogger.error(text);
            }
            break;
         case 2:
         case 3:
            cloudLogger && (text ? cloudLogger.info(text) : cloudLogger.info(msg));
            if (!onlyCloud) {
               text && consoleLogger.info(text);
            }
            break;
         case 4:
            cloudLogger && cloudLogger.warn(text);
            if (!onlyCloud) {
               consoleLogger.warn(text);
            }
            break;
      }
   }
};

function init(config, extendFn){
   if (!consoleLogger) {
      log4js.clearAppenders();
      log4js.addAppender(log4js.appenders.console(), 'inConsole');
      consoleLogger = log4js.getLogger('inConsole');
   }

   if (config.extra.cloudLogger && !cloudLogger) {
      sbisCloudLogger = require('sbis3-cloud-logger');
      sbisCloudLogger.init({
         cloud: config.log.address,                // Cloud hostname.
         logThreshold: 10,                         // How many log records to keep before sending.
         queueCheckTime: 5000,                     // How often to check log buffer.
         appName: config.appName || global.appName || 'sbis3-webrtc',// Application name (appears in log).
         appPort: config.extra.port                // Application port (appears in log).
      });

      log4js.addAppender(function (e) {
         var type = {20000: 'INFO', 30000: 'WARNING', 40000: 'ERROR'}[e.level.level];
         sbisCloudLogger.log(e.data[0], type);
      }, 'inCloud');

      cloudLogger = log4js.getLogger('inCloud');

      shutdownManager.addHandler(function () {
         if (sbisCloudLogger) {
            var d = Q.defer();
            sbisCloudLogger.end(function (err) {
               d[err ? 'reject' : 'resolve'](err);
            });
            return d.promise;
         }
      }, true);
   }

   if (typeof extendFn == "function"){
      var methods = extendFn(log4js);
      for (var i in methods){
         if (methods.hasOwnProperty(i)){
            methodsForExport[i] = methods[i];
         }
      }
   }

   return methodsForExport;
}

module.exports = init;
