var
      amqp = require('amqp'),
      os = require('os');

var
      getLogger = require('./lib/getLogger'),
      generateRoutingKey = require('./lib/generateRoutingKey');

var
      MAIN_EXCHANGE = '!web-entrypoint',
      SEND_LOG_ONLY_TO_CLOUD = true,
      SCOPE = 'user',
      SENDER = [os.hostname(), String(process.pid)];

var RabbitSender = function(config, logger, needServerConnection){
   this._cfg = config;
   this._ready = false;
   this._queue = [];
   this._logger = getLogger(logger);
   this._exchange = null;

   this._createClientConnection();
   
   if (needServerConnection) {
      this._createServerConnection();
   }
};

RabbitSender.prototype._send = function (userId, exchange, channel, data, cb) {
   var
         self = this,
         headers = {
            'event-type': channel,
            scope       : SCOPE,
            sender      : SENDER
         },
         _exchange = this._exchange;

   if (!exchange) {
      // если не передан обменник, будем паблишить во все из конфига
      headers.sites = this._cfg.exchanges;
   } else if (this._cfg.exchanges.indexOf(exchange) !== -1){
      // обменник есть в списке конфига
      headers.sites = [exchange];
   } else {
      // обменник не из конфига, его нужно создать, если он еще не создан
      _exchange = this._serverConnection.exchanges[exchange];
      if (_exchange) {
         if (_exchange.state !== 'open') {
            return this._queue.push(arguments);
         }
      } else {
         this._serverConnection.exchange(exchange, {
            passive: false,
            type: 'direct',
            confirm: true
         }, function(){
            self._logger.info('Rabbit Sender', 'Exchange "'+ exchange +'" opened!');
            self._processQueue();
         });
         return this._queue.push(arguments);
      }
   }

   if (_exchange) {
      var rk = generateRoutingKey(userId);
      _exchange.publish(generateRoutingKey(userId), data, {
         headers: headers,
         contentType: 'application/json; charset=utf-8'
      }, function () {
         //self._logger.info('Rabbit Sender', ['publish callback:', 'exchange: ' + _exchange.name, 'userId: ' + userId, 'rk: ' + rk, 'headers: ' + JSON.stringify(headers), 'data: ' + JSON.stringify(data)].join('\n'));
         typeof cb === 'function' && cb.apply(null, arguments);
      });
      this._logger.info('Rabbit Sender', ['publish:', 'exchange: ' + _exchange.name, 'userId: ' + userId, 'rk: ' + rk, 'headers: ' + JSON.stringify(headers), 'data: ' + JSON.stringify(data).slice(0, 200)].join('\n'), SEND_LOG_ONLY_TO_CLOUD);
   }
};

RabbitSender.prototype.send = function(/*userId, exchange, channel, data, cb*/){
   if (this._ready){
      this._send.apply(this, arguments);
   } else {
      this._queue.push(arguments);
   }
};

RabbitSender.prototype._processQueue = function () {
   if (this._ready) {
      while (this._queue.length) {
         this._send.apply(this, this._queue.shift());
      }
   }
};

RabbitSender.prototype._createConnection = function (address, connectionField, readyHandler) {
   var
         self = this,
         c;

   self._logger.info('Rabbit Sender', 'Trying connect to "' + address + '"');

   c = this[connectionField] = amqp.createConnection({url: address}, {
      reconnect: true,
      reconnectBackoffStrategy: 'exponential',
      reconnectBackoffTime: 1000,
      reconnectExponentialLimit: 3*60*60*1000 //3 часа
   });

   c.on('error', function (err) {
      self._logger.error('Rabbit Sender', err);
   });

   c.on('ready', function(){
      self._logger.info('Rabbit Sender', 'RabbitMQ on "' + address + '" is connected');
      typeof readyHandler === 'function' && readyHandler(c);
   });
};

RabbitSender.prototype._createServerConnection = function () {
   this._createConnection(this._cfg.serverAddress, '_serverConnection');
};

RabbitSender.prototype._createClientConnection = function () {
   var self = this;

   self._createConnection(this._cfg.clientAddress, '_clientConnection', function (connection) {
      self._exchange = connection.exchange(MAIN_EXCHANGE, {
         passive: true,
         confirm: true,
         type: 'fanout'
      }, function () {
         self._logger.info('Rabbit Sender', 'Exchange "'+ MAIN_EXCHANGE +'" opened!');
         self._ready = true;
         self._processQueue();
      });
   });
};

RabbitSender.prototype.getExchanges = function () {
   return this._cfg.exchanges;
};

function checkConfig (config){
   if(typeof(config) === 'undefined'){
      throw new Error('Rabbit sender init: config is undefined');
   }

   if(typeof(config.clientAddress) === 'undefined'){
      throw new Error('Rabbit sender init: config.clientAddress is undefined');
   }

   if(typeof(config.exchanges) === 'undefined'){
      throw new Error('Rabbit sender init: config.exchanges is undefined');
   }
}

function init (cfg, logger, needServerConnection) {
   checkConfig(cfg);
   return new RabbitSender(cfg, logger, needServerConnection);
}

init.Sender = RabbitSender;

module.exports = init;
