var
      Q = require('q');

var
      CODES = {
         SIGINT:            128 + 2,
         SIGTERM:           128 + 15,
         uncaughtException: 1
      },
      EXECUTION_TIMEOUT = 5 * 1000;

function isPromise (param) {
   return param && param.then && param.catch && typeof param.then === 'function' && typeof param.catch === 'function';
}

function ShutdownManager () {
   var self = this;

   this._handlers = [];
   this._lastHandlers = [];
   this._processing = false;

   process.on('uncaughtException', function (err) {
      self._exec(CODES.uncaughtException, err);
   });
   process.on('SIGINT', this._exec.bind(this, CODES.SIGINT));
   process.on('SIGTERM', this._exec.bind(this, CODES.SIGTERM));
   process.on('exit', function (exitCode) {
      self._exec(exitCode, new Error('ShutdownManager: unexpected exit code: ' + exitCode));
   });
}

ShutdownManager.prototype._exec = function (exitCode, err) {
   if (!this._processing) {
      this._processing = true;

      var
            self = this,
            result,
            promises = [],
            lastPromises = [],
            errors = [];

      var

            exit = function () {
               self._exit(exitCode, errors);
            },
            allWithTimeout = function (promiseArray) {
               return Q.all(promiseArray).timeout(EXECUTION_TIMEOUT);
            },
            allLastPromises = function () {
               self._lastHandlers.forEach(function (hdl) {
                  try {
                     if (isPromise(result = hdl(err, exitCode))) {
                        lastPromises.push(result);
                     }
                  } catch (e) {
                     errors.push(e);
                  }
               });

               allWithTimeout(lastPromises)
                     .then(exit)
                     .catch(function (error) {
                        errors.push(error);
                        exit();
                     });
            };

      this._handlers.forEach(function (hdl) {
         try {
            if (isPromise(result = hdl(err, exitCode))) {
               promises.push(result);
            }
         } catch (e) {
            errors.push(e);
         }
      });

      allWithTimeout(promises)
            .then(function () {
               allLastPromises();
            })
            .catch(function (error) {
               errors.push(error);
               allLastPromises();
            });
   }
};

ShutdownManager.prototype._exit = function (exitCode) {
   process.exit(exitCode);
};

ShutdownManager.prototype.addHandler = function (hdl, last) {
   if (hdl && typeof hdl === 'function') {
      this[last ? '_lastHandlers' : '_handlers'].push(hdl);
   }
};

module.exports = new ShutdownManager();