var
      amqp = require('amqp'),
      inherits = require('util').inherits,
      EventEmitter = require('events').EventEmitter,
      os = require('os');

var
      getLogger = require('./lib/getLogger'),
      generateRoutingKey = require('./lib/generateRoutingKey');

var RabbitSubscriber = function (cfg, logger) {
   EventEmitter.call(this);
   this._address = cfg.serverAddress;
   this._connection = null;
   this._ready = false;
   this._queue = null;
   this._logger = getLogger(logger);

   this._connect();
};

inherits(RabbitSubscriber, EventEmitter);

RabbitSubscriber.prototype.subscribe = function (exchange, id) {
   this._exchange = exchange;
   this._id = id;
   if (this._ready) {
      this._subscribe();
   }
   // всегда при новом подключении к amqp серверу будет пересоздавать очередь
   this._connection.on('ready', this._subscribe.bind(this));
};

RabbitSubscriber.prototype._subscribe = function () {
   var self = this;
   this._connection.exchange(this._exchange, {
      type: 'direct'
   }, function(){
      self._logger.info('RabbitSubscriber', 'Exchange "'+ self._exchange +'" created!');
      self._connection.queue(os.hostname() + '_' + process.pid, {
         exclusive: true
      }, function (queue) {
         self._logger.info('RabbitSubscriber', ' Create queue: exchange: ' + self._exchange + '; id: ' + self._id);
         self._queue = queue;
         queue.bind(self._exchange, self._getRoutingKey(), function () {
            self._logger.info('RabbitSubscriber', 'Bind queue to exchange: ' + self._exchange);
            queue.subscribe(function (message, headers) {
               self.emit('message', headers['event-type'], message);
            });
            self.emit('ready');
         });
      });
   });
};

RabbitSubscriber.prototype._connect = function () {
   var self = this;

   self._logger.info('RabbitSubscriber', 'Trying connect to ' + this._address);

   this._connection = amqp.createConnection({url: this._address}, {
      reconnect: true,
      reconnectBackoffStrategy: 'exponential',
      reconnectBackoffTime: 1000,
      reconnectExponentialLimit: 3 * 60 * 60 * 1000 //3 часа
   });

   this._connection.on('ready', function () {
      self._logger.info('RabbitSubscriber', 'Create connection to ' + self._address);
      self._ready = true;
   });

   this._connection.on('error', function (err) {
      self._logger.error('RabbitSubscriber', 'Connection on error: ' + err);
   });
};

RabbitSubscriber.prototype._getRoutingKey = function () {
   return generateRoutingKey(this._id);
};

function checkConfig (config){
   if(typeof(config) === 'undefined'){
      throw new Error('Rabbit subscriber init: config is undefined');
   }

   if(typeof(config.serverAddress) === 'undefined'){
      throw new Error('Rabbit subscriber init: config.serverAddress is undefined');
   }
}

function init (cfg, logger) {
   checkConfig(cfg);
   return new RabbitSubscriber(cfg, logger);
}

init.Subscriber = RabbitSubscriber;

module.exports = init;