let request = require('request');
let cash = {};

const CASH_TTL = 1000*60*10;

function getBuildNumber(host, sid) {
   const getServiceBuildNumber = function getServiceBuildNumber(serviceUrl = '') {
      return new Promise((resolve, reject) => {
         request(
            {
               url: `http://${host}${serviceUrl}/resources/contents.json`,
               headers: {
                  cookie: `sid=${sid}`
               }
            },
            function(err, res) {
               if (err) {
                  reject(err);
               } else {
                  try {
                     resolve(JSON.parse(res.body).buildnumber);
                  } catch (error) {
                     reject(error);
                  }
               }
            });
      })
   };
   const getMainServiceBuildNumber = getServiceBuildNumber,
         getVideocallServiceBuildNumber = getServiceBuildNumber.bind(null, '/videocall');

   if (cash[host] && new Date() - cash[host].datetime < CASH_TTL) {
      return Promise.resolve(cash[host].buildnumber);
   } else {
      return Promise.all([
         getMainServiceBuildNumber(),
         getVideocallServiceBuildNumber()
      ]).then(([mainServiceBuildNumber, videocallServiceBuildNumber]) => {
         const mainServiceBuildNumberAsNumber      = parseInt(mainServiceBuildNumber, 10),
               videocallServiceBuildNumberAsNumber = parseInt(videocallServiceBuildNumber, 10),
               buildnumber = mainServiceBuildNumberAsNumber > videocallServiceBuildNumberAsNumber || isNaN(videocallServiceBuildNumberAsNumber) ?
                  mainServiceBuildNumber : videocallServiceBuildNumber;

         // взяли версию сервиса, обновившегося последним
         cash[host] = {
            buildnumber,
            datetime: new Date()
         };
      });
   }
}

module.exports = getBuildNumber;







